//
//  DownloadCell.swift
//  AmpSimpleSample
//
//  Created by Miguel Hernandez on 1/11/19.
//  Copyright © 2023 Akamai. All rights reserved.
//

import UIKit
import AmpDownloader

class DownloadCell:UICollectionViewCell, AmpDownloaderDelegate {
    
    func downloadPaused(asset: AmpDownload) {
        if (download?.id == asset.id) {
            toggleBtn.setTitle("Resume", for: .normal)
        }
    }
    
    func downloadResumed(asset: AmpDownload) {
        if (download?.id == asset.id) {
            toggleBtn.setTitle("Pause", for: .normal)
        }
    }
    
    func downloadCompleted(asset: AmpDownload) {
        if (download?.id == asset.id) {
            progressBar.tintColor = UIColor.green
            progressBar.setProgress(1.0, animated: true)
            progress.text = "\(100)%"
            deleteBtn.setTitle("Delete", for: .normal)
            enableToggleBtn(enabled: false)
        }
    }
    
    func downloadProgressChanged(asset: AmpDownload, progress: Float) {
        if (download?.id == asset.id) {
            progressBar.setProgress(progress, animated: true)
            self.progress.text = "\(Int(progress * 100))%"
        }
    }
    

    @IBOutlet weak var title: UILabel!
    @IBOutlet weak var progress: UILabel!
    @IBOutlet weak var progressBar: UIProgressView!
    @IBOutlet weak var toggleBtn: UIButton!
    @IBOutlet weak var deleteBtn: UIButton!
    
    
    
    @IBAction func onDelete(_ sender: Any) {
        guard let vodDownload = download else {
            return
        }
        
        if vodDownload.isCompleted {
            vodDownload.deleteVideo()
        } else {
            AmpDownloadManager.cancelDownload(for: vodDownload)
        }
    }
    
    @IBAction func onToggle(_ sender: Any) {
        guard let vodDownload = download else {
            return
        }
        print("toggling \(vodDownload.id)")
        print ("Jairo ----------")
        if vodDownload.isPaused {
            AmpDownloadManager.resumeDownload(for: vodDownload)
        } else {
            AmpDownloadManager.pauseDownload(for: vodDownload)
        }
    }
    
    private var download:AmpDownload?
    
    func setForDownload(download:AmpDownload) {
        AmpDownloadManager.registerDelegate(delegate: self)
        print("registering \(download.id)")
        self.download = download
        title.text = download.title
        
        if download.isCompleted {
            downloadCompleted(asset: download)
        } else {
            progressBar.setProgress(0.0, animated: false)
            progress.text = "\(0)%"
            enableToggleBtn(enabled: true)
        }
        
        
    }
    
    private func enableToggleBtn(enabled:Bool) {
        guard let vodDownload = download else {
            return
        }
        
        if enabled {
            toggleBtn.isEnabled = true
            toggleBtn.setTitle(vodDownload.isPaused ? "Resume" : "Pause", for: .normal)
        } else {
            toggleBtn.isEnabled = false
            toggleBtn.setTitle("", for: .normal)
        }
    }
}
