//
//  DownloadVC.swift
//  AmpSimpleSample
//
//  Created by Miguel Hernandez on 1/11/19.
//  Copyright © 2023 Akamai. All rights reserved.
//

import UIKit
import AVKit
import AmpDownloader

class DownloadVC: UIViewController {
    
    @IBOutlet weak var collectionView: UICollectionView!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        collectionView.register(UINib(nibName: "DownloadCell", bundle: nil), forCellWithReuseIdentifier: "DownloadCell")
        AmpDownloadManager.registerDelegate(delegate: self)
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "Play" {
            let cell = sender as! DownloadCell
            let indexPaths = collectionView.indexPath(for: cell)
            let download = AmpDownloadManager.getDownloadList()[indexPaths!.row]
            (segue.destination as! LocalPlayerVC).download = download
        }
    }
}

extension DownloadVC: AmpDownloaderDelegate {
    public func downloadCancelled(asset: AmpDownload) {
        collectionView.reloadData()
        print("The download was cancelled")
    }
    
    public func downloadWillDelete(asset: AmpDownload) {
        print("The download will delete")
    }
    
    public func downloadPaused(asset: AmpDownload) {
        print("The download is paused")
    }
    
    public func downloadResumed(asset: AmpDownload) {
        print("The download is resumed")
    }
    
    
    public func downloadStarted(asset: AmpDownload) {
        print("The download has started")
        collectionView.reloadData()
    }
    
    public func downloadCompleted(asset: AmpDownload) {
        print("The download has completed")
    }
    
    public func downloadProgressChanged(asset: AmpDownload, progress: Float) {
        print("The progress is in value \(progress)")
    }
    
    func downloadFailed(asset: AmpDownload, error: NSError, message: String) {
        print ("An error has occurred: \(error) with message: \(message)")
    }
    
}

extension DownloadVC: UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return AmpDownloadManager.getDownloadList().count
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "DownloadCell", for: indexPath) as! DownloadCell
        cell.setForDownload(download: AmpDownloadManager.getDownloadList()[indexPath.row])
        return cell
    }
}

extension DownloadVC: UICollectionViewDelegateFlowLayout {
    func collectionView(_ collectionView: UICollectionView, layout collectionViewLayout: UICollectionViewLayout, sizeForItemAt indexPath: IndexPath) -> CGSize {
        let kWhateverHeightYouWant = 91
        return CGSize(width: collectionView.bounds.size.width, height: CGFloat(kWhateverHeightYouWant))
    }
}

extension DownloadVC: UICollectionViewDelegate {
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        if let cell = collectionView.cellForItem(at: indexPath) {
            performSegue(withIdentifier: "Play", sender: cell)
        }
    }
    
}
