//
//  AmpWritter.swift
//  AmpDownloader
//
//  Created by Miguel Hernandez on 3/15/19.
//  Copyright © 2019 Miguel Hernandez. All rights reserved.
//

import Foundation

public class AmpWritter {
    
    static let instance = AmpWritter()
    
    var ignoreUpdate = false
    
    var downloadsArray:[AmpDownload]! {
        didSet {
            if ignoreUpdate {
                ignoreUpdate = false
            } else {
                let encoder = PropertyListEncoder()
                encoder.outputFormat = .xml
                do {
                    let data = try encoder.encode(downloadsArray)
                    try data.write(to: NSURL.fileURL(withPath:path))
                } catch {
                    print (error)
                }
            }
        }
    }
    
    let path:String = {
        let documentDirectory = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0] as String
        return documentDirectory.appending("/AmpDownloads.plist")
    }()
    
    private init() {
        if(!FileManager.default.fileExists(atPath: path)){
            let emptyData = NSMutableArray()
            let _ = emptyData.write(toFile: path, atomically: true)
        }
        
        ignoreUpdate = true
        let _ = getDownloadData(forceUpdate: true)
    }
    
    private func getDownloadData(forceUpdate: Bool, withCanceledDownloads:Bool = false) -> [AmpDownload] {
        if downloadsArray != nil && !forceUpdate {
            return downloadsArray
        }
        
        var array:[AmpDownload] = []
        if let data = FileManager.default.contents(atPath: path),
            let decodedArray = try? PropertyListDecoder().decode(Array<AmpDownload>.self, from: data) {
            array = decodedArray
        } else {
            array = []
        }
        
        downloadsArray = array
        return downloadsArray.filter {
            withCanceledDownloads ? true : $0._isCanceled == false
        }
    }
    
    internal static func getDownloadData(forceUpdate: Bool) -> [AmpDownload] {
        return instance.getDownloadData(forceUpdate: forceUpdate)
    }
    
    internal static func removeCanceledDownloads() {
        instance.downloadsArray = instance.downloadsArray.filter {
            $0._isCanceled == false
        }
    }
    
    internal static func contains(downloadId:String) -> Bool {
        return instance.downloadsArray.contains { $0._id == downloadId }
    }
    
    internal static func contains(download: AmpDownload) -> Bool {
        return instance.downloadsArray.contains { $0 == download }
    }
    
    internal static func getDownload(id:String, withCanceledDownloads:Bool = false) -> AmpDownload? {
        let results = instance.downloadsArray.filter {
            $0._id == id && (withCanceledDownloads ? true : $0._isCanceled == false)
        }
        if let download = results.first, results.count == 1 {
            return download
        } else {
            return nil
        }
    }
    
    internal static func insertDownload(download:AmpDownload) -> Bool {
        var downloads = instance.downloadsArray
        downloads?.append(download)
        downloads?.sort {$0._id < $1._id}
        instance.downloadsArray = downloads
        return true
    }
    
    internal static func updateDownload(download:AmpDownload) -> Bool {
        var downloads = instance.downloadsArray
        downloads?.append(download)
        downloads?.sort {$0._id < $1._id}
        instance.downloadsArray = downloads
        return true
    }
    
    internal static func deleteDownload(index:Int) -> Bool {
        if instance.downloadsArray.count > index && index >= 0 {
            instance.downloadsArray[index].deleteDownload()
            instance.downloadsArray[index]._isCanceled = true
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
    internal static func deleteDownload(download:AmpDownload) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            return deleteDownload(index: index)
        } else {
            return false
        }
    }
    
    internal static func setDownloadLocation(id:String, location:URL) -> Bool {
        if let download = getDownload(id: id) {
            return setDownloadLocation(download:download, location:location)
        } else {
            return false
        }
    }
    
    internal static func setDownloadLocation(download:AmpDownload, location:URL) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            instance.downloadsArray[index]._location = location
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
    internal static func pauseDownload(download:AmpDownload) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            instance.downloadsArray[index]._isPaused = true
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
    internal static func resumeDownload(download:AmpDownload) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            instance.downloadsArray[index]._isPaused = false
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
    internal static func failDownload(download:AmpDownload) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            instance.downloadsArray[index]._isCompleted = false
            instance.downloadsArray[index]._isPaused = false
            instance.downloadsArray[index]._isFailed = true
            instance.downloadsArray[index]._location = nil
            instance.downloadsArray[index]._progress = 0.0
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
    internal static func completeDownload(download:AmpDownload) -> Bool {
        if let index = instance.downloadsArray.firstIndex(of: download) {
            instance.downloadsArray[index]._isCompleted = true
            instance.downloadsArray[index]._isPaused = false
            instance.downloadsArray[index]._isFailed = false
            instance.downloadsArray[index]._isCanceled = false
            instance.downloadsArray[index]._progress = 1.0
            instance.downloadsArray = instance.downloadsArray
            return true
        } else {
            return false
        }
    }
    
}
